import { __spreadProps, __spreadValues } from '../chunk-5USKE2QT.js';
import { useMemo, useRef, useState } from 'react';
import { Transition } from 'react-transition-preset';
import useIsomorphicLayoutEffect from '../hooks/useIsomorphicLayoutEffect.js';
import { classnames } from '../utils.js';
import { jsx } from 'react/jsx-runtime';

function Toast(props) {
  const {
    content,
    className,
    duration,
    onClosed,
    pauseOnHover,
    hover,
    updateFlag,
    transition: _transition,
    open: _open,
    onEnter: _onEnter,
    onExit: _onExit,
    offsetHeight
  } = props;
  const transition = useMemo(() => {
    if (typeof _transition === "string") {
      return { transition: _transition };
    }
    return {
      transition: _transition == null ? void 0 : _transition.name,
      duration: _transition == null ? void 0 : _transition.duration,
      exitDuration: _transition == null ? void 0 : _transition.exitDuration
    };
  }, [_transition]);
  const timer = useRef();
  const ref = useRef(null);
  const [open, setOpen] = useState(!!_open);
  useIsomorphicLayoutEffect(() => {
    if (typeof _open === "boolean") {
      setOpen(!!_open);
    }
  }, [_open]);
  const delayClear = () => {
    if (duration) {
      timer.current && clearTimeout(timer.current);
      timer.current = window.setTimeout(() => {
        setOpen(false);
      }, +duration);
    }
  };
  const onMouseHover = (hover2) => {
    if (!pauseOnHover || !open) return;
    if (hover2) {
      timer.current && clearTimeout(timer.current);
    } else {
      delayClear();
    }
  };
  useIsomorphicLayoutEffect(() => {
    onMouseHover(hover);
  }, [hover]);
  const onEnter = () => {
    if (!ref.current) return;
    const height = ref.current.clientHeight;
    _onEnter(height);
  };
  const onExit = () => {
    var _a;
    const height = (_a = ref.current) == null ? void 0 : _a.clientHeight;
    _onExit(height || 0);
  };
  const onExited = () => {
    onClosed == null ? void 0 : onClosed();
  };
  const resolveTransform = (transform, offsetHeight2) => {
    const internalTransform = `translate(-50%, calc(-50% - ${offsetHeight2}px))`;
    return [internalTransform, transform].filter(Boolean).join(" ");
  };
  const resolveTransformProperty = (transform) => {
    return [.../* @__PURE__ */ new Set(["transform", transform])].filter(Boolean).join(", ");
  };
  useIsomorphicLayoutEffect(() => {
    delayClear();
    return () => {
      timer.current && clearTimeout(timer.current);
    };
  }, [updateFlag]);
  useIsomorphicLayoutEffect(() => {
    if (!content) {
      onExited();
    }
  }, [content]);
  return /* @__PURE__ */ jsx(Transition, __spreadProps(__spreadValues({ mounted: open }, transition), { onEnter, onExit, onExited, initial: true, children: (styles) => /* @__PURE__ */ jsx(
    "div",
    {
      style: __spreadProps(__spreadValues({}, styles), {
        transitionProperty: resolveTransformProperty(styles.transitionProperty),
        transform: resolveTransform(styles.transform, offsetHeight)
      }),
      ref,
      className: classnames("toast__content", className),
      children: content
    }
  ) }));
}
var toast_default = Toast;

export { toast_default as default };
