'use strict';

var React = require('react');
var reactTransitionPreset = require('react-transition-preset');
var jsxRuntime = require('react/jsx-runtime');
var client = require('react-dom/client');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var React__default = /*#__PURE__*/_interopDefault(React);

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);

// src/utils.ts
function isBrowser() {
  return typeof window !== "undefined" && typeof document !== "undefined";
}
function omit(obj, keys) {
  const copy = __spreadValues({}, obj);
  keys.forEach((key) => delete copy[key]);
  return copy;
}
var classnames = (...args) => args.filter(Boolean).join(" ");
var useIsomorphicLayoutEffect = typeof window !== "undefined" ? React__default.default.useLayoutEffect : React__default.default.useEffect;
var useIsomorphicLayoutEffect_default = useIsomorphicLayoutEffect;
function Toast(props) {
  const {
    content,
    className,
    duration,
    onClosed,
    pauseOnHover,
    hover,
    updateFlag,
    transition: _transition,
    open: _open,
    onEnter: _onEnter,
    onExit: _onExit,
    offsetHeight
  } = props;
  const transition = React.useMemo(() => {
    if (typeof _transition === "string") {
      return { transition: _transition };
    }
    return {
      transition: _transition == null ? void 0 : _transition.name,
      duration: _transition == null ? void 0 : _transition.duration,
      exitDuration: _transition == null ? void 0 : _transition.exitDuration
    };
  }, [_transition]);
  const timer = React.useRef();
  const ref = React.useRef(null);
  const [open, setOpen] = React.useState(!!_open);
  useIsomorphicLayoutEffect_default(() => {
    if (typeof _open === "boolean") {
      setOpen(!!_open);
    }
  }, [_open]);
  const delayClear = () => {
    if (duration) {
      timer.current && clearTimeout(timer.current);
      timer.current = window.setTimeout(() => {
        setOpen(false);
      }, +duration);
    }
  };
  const onMouseHover = (hover2) => {
    if (!pauseOnHover || !open) return;
    if (hover2) {
      timer.current && clearTimeout(timer.current);
    } else {
      delayClear();
    }
  };
  useIsomorphicLayoutEffect_default(() => {
    onMouseHover(hover);
  }, [hover]);
  const onEnter = () => {
    if (!ref.current) return;
    const height = ref.current.clientHeight;
    _onEnter(height);
  };
  const onExit = () => {
    var _a;
    const height = (_a = ref.current) == null ? void 0 : _a.clientHeight;
    _onExit(height || 0);
  };
  const onExited = () => {
    onClosed == null ? void 0 : onClosed();
  };
  const resolveTransform = (transform, offsetHeight2) => {
    const internalTransform = `translate(-50%, calc(-50% - ${offsetHeight2}px))`;
    return [internalTransform, transform].filter(Boolean).join(" ");
  };
  const resolveTransformProperty = (transform) => {
    return [.../* @__PURE__ */ new Set(["transform", transform])].filter(Boolean).join(", ");
  };
  useIsomorphicLayoutEffect_default(() => {
    delayClear();
    return () => {
      timer.current && clearTimeout(timer.current);
    };
  }, [updateFlag]);
  useIsomorphicLayoutEffect_default(() => {
    if (!content) {
      onExited();
    }
  }, [content]);
  return /* @__PURE__ */ jsxRuntime.jsx(reactTransitionPreset.Transition, __spreadProps(__spreadValues({ mounted: open }, transition), { onEnter, onExit, onExited, initial: true, children: (styles) => /* @__PURE__ */ jsxRuntime.jsx(
    "div",
    {
      style: __spreadProps(__spreadValues({}, styles), {
        transitionProperty: resolveTransformProperty(styles.transitionProperty),
        transform: resolveTransform(styles.transform, offsetHeight)
      }),
      ref,
      className: classnames("toast__content", className),
      children: content
    }
  ) }));
}
var toast_default = Toast;
function ToastContainer(props) {
  const { toasts, queue } = props;
  const [hoverState, setHoverState] = React.useState(false);
  const [heightMap, setHeightMap] = React.useState(/* @__PURE__ */ new Map());
  const onEnter = (key, height) => {
    setHeightMap((prev) => {
      prev.set(key, height);
      return new Map(prev);
    });
  };
  const onExit = (key) => {
    setHeightMap((prev) => {
      prev.delete(key);
      return new Map(prev);
    });
  };
  const offsetHeight = (_toast, index) => {
    let offset = 0;
    for (let i = toasts.length - 1; i > index; i--) {
      offset += (heightMap.get(toasts[i].key) || 0) + toasts[i].gap;
    }
    return offset;
  };
  return /* @__PURE__ */ jsxRuntime.jsx(
    "div",
    {
      onMouseEnter: () => {
        setHoverState(true);
      },
      onMouseLeave: () => {
        setHoverState(false);
      },
      className: "toast__container",
      children: toasts.map((toast2, index) => /* @__PURE__ */ jsxRuntime.jsx(
        toast_default,
        __spreadValues({
          onEnter: (size) => {
            onEnter(toast2.key, size);
          },
          onExit: () => {
            onExit(toast2.key);
          },
          onClosed: () => {
            var _a;
            (_a = toast2.onClosed) == null ? void 0 : _a.call(toast2);
            queue.remove(toast2.key);
          },
          hover: hoverState,
          offsetHeight: offsetHeight(toast2, index)
        }, omit(toast2, ["key"])),
        toast2.key
      ))
    }
  );
}
var toast_container_default = ToastContainer;
var MARK = "__react_root__";
function mount(node, container) {
  const root = container[MARK] || client.createRoot(container);
  root.render(node);
  container[MARK] = root;
}
var Renderer = class {
  constructor(queue) {
    __publicField(this, "containerID", "react-atom-toast");
    __publicField(this, "container", null);
    __publicField(this, "queue");
    this.queue = queue;
    if (isBrowser()) {
      let container = document.getElementById(this.containerID);
      if (!container) {
        container = document.createElement("div");
        container.id = this.containerID;
        document.body.appendChild(container);
      }
      this.container = container;
    }
  }
  render(toasts) {
    if (!this.container) {
      return;
    }
    mount(/* @__PURE__ */ jsxRuntime.jsx(toast_container_default, { toasts, queue: this.queue }), this.container);
  }
};

// src/toast-queue.ts
var ToastQueue = class {
  constructor() {
    __publicField(this, "renderer");
    __publicField(this, "toasts", []);
    this.renderer = new Renderer(this);
  }
  render() {
    this.renderer.render(this.toasts);
  }
  add(options) {
    var _a;
    const { maxCount, key } = options;
    if (key && this.toasts.some((t) => t.key === key)) {
      this.update(key, options);
      return;
    }
    if (maxCount && this.toasts.length >= maxCount) {
      this.close((_a = this.toasts.filter((t) => t.open === true)[0]) == null ? void 0 : _a.key);
    }
    const toastOptions = __spreadProps(__spreadValues({}, options), {
      key: options.key || Math.random().toString(36),
      open: true
    });
    this.toasts.push(toastOptions);
    this.render();
  }
  close(key) {
    if (!key) return;
    this.update(key, { open: false });
    this.render();
  }
  closeAll() {
    this.toasts = this.toasts.map((toastOptions) => __spreadProps(__spreadValues({}, toastOptions), {
      open: false
    }));
    this.render();
  }
  remove(key) {
    this.toasts = this.toasts.filter((toastOptions) => toastOptions.key !== key);
    this.render();
  }
  removeAll() {
    this.toasts = [];
    this.render();
  }
  update(key, options) {
    const index = this.toasts.findIndex((toastOptions) => toastOptions.key === key);
    if (index !== -1) {
      this.toasts[index] = __spreadProps(__spreadValues(__spreadValues({}, this.toasts[index]), options), {
        updateFlag: !this.toasts[index].updateFlag
      });
    }
    this.render();
  }
};

// src/index.ts
var Toast2 = class {
  constructor() {
    __publicField(this, "defaultOptions", {
      duration: 2e3,
      pauseOnHover: true,
      transition: "fade",
      maxCount: 3,
      gap: 16
    });
    __publicField(this, "toastQueue");
    this.toastQueue = new ToastQueue();
  }
  open(options) {
    this.toastQueue.add(__spreadValues(__spreadValues({}, this.defaultOptions), options));
  }
  close(key) {
    this.toastQueue.close(key);
  }
  closeAll() {
    this.toastQueue.closeAll();
  }
  update(key, options) {
    this.toastQueue.update(key, __spreadValues(__spreadValues({}, this.defaultOptions), options));
  }
  setDefaultOptions(options) {
    this.defaultOptions = __spreadValues(__spreadValues({}, this.defaultOptions), options);
  }
};
var toast = new Toast2();

exports.toast = toast;
